import time
import torch
import logging
from modelscope import AutoModelForCausalLM, AutoTokenizer

# Setup logger
logging.basicConfig(filename='Qwen2.5_Inference_Batching_SuperComputer4090_log.txt', level=logging.INFO)

# Model and tokenizer initialization
model_name = "/mnt/export/home/nsccwx_zhangwei1/zhangwei1/data/benchmark/ahead05/qwen/Qwen2___5-7B-Instruct"

model = AutoModelForCausalLM.from_pretrained(
    model_name,
    torch_dtype="auto",
    device_map="auto",
)
tokenizer = AutoTokenizer.from_pretrained(model_name, padding_side="left")

# Prepare input message batch
message_batch = [
    [{"role": "user", "content": "你能介绍一下人工智能吗？"}],
    [{"role": "user", "content": "What is the capital of France?"}],
    [{"role": "user", "content": "你喜欢看电影吗？"}],
    [{"role": "user", "content": "How do I get better at coding?"}],
    [{"role": "user", "content": "你能给我讲讲机器学习的基本概念吗？"}],
    [{"role": "user", "content": "What is the meaning of life?"}],
    [{"role": "user", "content": "你认为科技会如何改变我们的生活？"}],
    [{"role": "user", "content": "Can you explain the concept of quantum computing?"}],
    [{"role": "user", "content": "你喜欢什么类型的音乐？"}],
    [{"role": "user", "content": "How does climate change affect the environment?"}],
    [{"role": "user", "content": "你能教我如何学习一门新语言吗？"}],
    [{"role": "user", "content": "What is the difference between a virus and bacteria?"}],
    [{"role": "user", "content": "你觉得未来的工作会是什么样子？"}],
    [{"role": "user", "content": "What is artificial intelligence and how does it work?"}],
    [{"role": "user", "content": "你喜欢旅行吗？"}],
    [{"role": "user", "content": "What are the main challenges in space exploration?"}],
    [{"role": "user", "content": "你最喜欢的书是什么？"}],
    [{"role": "user", "content": "What is the best way to stay healthy?"}],
    [{"role": "user", "content": "你怎么看待现代教育的改变？"}],
    [{"role": "user", "content": "What are the benefits of meditation?"}],
    [{"role": "user", "content": "你如何看待人工智能的未来发展？"}],
    [{"role": "user", "content": "What is the difference between renewable and non-renewable energy?"}],
    [{"role": "user", "content": "你喜欢哪些运动？"}],
    [{"role": "user", "content": "How do you build a successful business?"}],
    [{"role": "user", "content": "你认为什么是健康的生活方式？"}],
    [{"role": "user", "content": "How do black holes work?"}],
    [{"role": "user", "content": "你能介绍一下深度学习的基本概念吗？"}],
    [{"role": "user", "content": "What is the best way to learn a new language?"}],
    [{"role": "user", "content": "你如何理解量子力学？"}],
    [{"role": "user", "content": "What is the meaning of the term 'blockchain'?"}],
    [{"role": "user", "content": "你认为未来的交通工具会是什么样子？"}],
    [{"role": "user", "content": "What are the key principles of sustainable development?"}],
    [{"role": "user", "content": "你怎么看待全球变暖问题？"}],
    [{"role": "user", "content": "What is the significance of the human genome project?"}],
    [{"role": "user", "content": "你喜欢喝咖啡吗？"}],
    [{"role": "user", "content": "How do you manage stress effectively?"}],
    [{"role": "user", "content": "你怎么看待人工智能在医疗领域的应用？"}],
    [{"role": "user", "content": "What is the role of artificial intelligence in modern technology?"}],
    [{"role": "user", "content": "你对未来的能源技术有什么看法？"}],
    [{"role": "user", "content": "What are some key strategies for effective time management?"}],
    [{"role": "user", "content": "你认为未来五年内哪些技术会得到快速发展？"}],
    [{"role": "user", "content": "What are the main challenges of remote work?"}],
    [{"role": "user", "content": "你觉得现在的互联网对我们的影响有多大？"}],
    [{"role": "user", "content": "What is the role of data in today's world?"}],
    [{"role": "user", "content": "你认为人工智能能完全取代人类工作吗？"}],
    [{"role": "user", "content": "How can we reduce the impact of plastic pollution?"}],
    [{"role": "user", "content": "你认为科技公司如何平衡创新和道德问题？"}],
    [{"role": "user", "content": "What is the importance of cybersecurity in the digital age?"}],
    [{"role": "user", "content": "你认为人类会在太空中生活吗？"}],
    [{"role": "user", "content": "What are the primary factors contributing to global poverty?"}],
    [{"role": "user", "content": "你最喜欢的电影是什么？"}],
    [{"role": "user", "content": "How does the internet of things (IoT) work?"}],
    [{"role": "user", "content": "你怎么看待基因编辑技术？"}],
    [{"role": "user", "content": "What are the most important skills for success in the modern workforce?"}],
    [{"role": "user", "content": "你认为人工智能对教育的影响是什么？"}],
    [{"role": "user", "content": "What are the main ethical considerations in artificial intelligence?"}],
    [{"role": "user", "content": "你如何定义成功？"}],
    [{"role": "user", "content": "How can we improve public transportation systems?"}],
    [{"role": "user", "content": "你认为人类的未来会是什么样子？"}],
    [{"role": "user", "content": "What is the importance of a strong work-life balance?"}],
    [{"role": "user", "content": "你喜欢看书吗？"}],
    [{"role": "user", "content": "How can artificial intelligence enhance healthcare services?"}],
    [{"role": "user", "content": "你觉得将来的互联网会变得更智能吗？"}],
    [{"role": "user", "content": "What are the most significant achievements in space exploration?"}],
    [{"role": "user", "content": "你如何看待网络隐私问题？"}],
    [{"role": "user", "content": "What are the key differences between traditional and digital marketing?"}],
    [{"role": "user", "content": "你能教我如何提高我的写作技能吗？"}],
    [{"role": "user", "content": "What is the impact of social media on society?"}],
    [{"role": "user", "content": "你怎么看待人工智能对个人隐私的威胁？"}],
    [{"role": "user", "content": "What are the major environmental challenges facing the world today?"}],
    [{"role": "user", "content": "你认为人工智能的进步是否会带来伦理问题？"}],
    [{"role": "user", "content": "What is the future of autonomous vehicles?"}],
    [{"role": "user", "content": "你认为智能家居会变得更加普及吗？"}],
    [{"role": "user", "content": "What are some of the benefits of renewable energy sources?"}],
    [{"role": "user", "content": "你如何看待现代社会中人类与科技的关系？"}],
    [{"role": "user", "content": "How do we address the growing problem of inequality?"}],
    [{"role": "user", "content": "你认为未来的科技会如何影响我们的工作方式？"}],
    [{"role": "user", "content": "What is the importance of digital literacy in today's world?"}],
    [{"role": "user", "content": "你对未来的互联网有什么期望？"}],
    [{"role": "user", "content": "How does blockchain technology work?"}],
    [{"role": "user", "content": "你觉得网络游戏的影响是什么？"}],
    [{"role": "user", "content": "What are the ethical issues surrounding artificial intelligence?"}],
    [{"role": "user", "content": "你怎么看待未来的教育方式？"}],
    [{"role": "user", "content": "What is the potential impact of 5G technology on communication?"}],
    [{"role": "user", "content": "你喜欢哪种类型的食物？"}],
    [{"role": "user", "content": "What is the role of education in personal development?"}],
    [{"role": "user", "content": "你认为人工智能在未来几年会发展到什么程度？"}]
]
text_batch = tokenizer.apply_chat_template(
    message_batch,
    tokenize=False,
    add_generation_prompt=True,
)

# Tokenize the input
start_tokenization_time = time.time()
model_inputs_batch = tokenizer(text_batch, return_tensors="pt", padding=True).to(model.device)
tokenization_end_time = time.time()

# Start measuring generation time
start_gen_time = time.time()
generated_ids_batch = model.generate(
    **model_inputs_batch,
    max_new_tokens=512,
)
generation_end_time = time.time()

# Post-processing: Decode generated tokens
generated_ids_batch = generated_ids_batch[:, model_inputs_batch.input_ids.shape[1]:]
response_batch = tokenizer.batch_decode(generated_ids_batch, skip_special_tokens=True)

# Calculate performance metrics
first_token_latency = generation_end_time - start_gen_time  # Time for the first token
total_time = generation_end_time - start_tokenization_time  # Total end-to-end time
tokens_generated = generated_ids_batch.shape[1]  # Number of tokens generated

# Per-token latency
per_token_latency = (generation_end_time - start_gen_time) / tokens_generated if tokens_generated > 0 else 0

# Throughput (tokens per second)
throughput = tokens_generated / total_time if total_time > 0 else 0

# GPU memory usage
memory_allocated = torch.cuda.memory_allocated(model.device) / (1024 ** 3) / 24 # in GB %

# Power consumption (using pynvml or nvidia-smi)
# Placeholder function to track power usage
def get_gpu_power_usage():
    try:
        import pynvml
        pynvml.nvmlInit()
        handle = pynvml.nvmlDeviceGetHandleByIndex(0)  # assuming 0 is the GPU index
        power_usage = pynvml.nvmlDeviceGetPowerUsage(handle) / 1000  # in watts
        pynvml.nvmlShutdown()
        return power_usage
    except Exception as e:
        print(f"Error retrieving power usage: {e}")
        return None

power_usage = get_gpu_power_usage()

# Log performance metrics
logging.info("Performance Metrics:")
logging.info(f"Generated Responses: {response_batch}")
logging.info(f"First Token Latency: {first_token_latency:.4f} seconds")
logging.info(f"Per Token Latency: {per_token_latency:.4f} seconds")
logging.info(f"End-to-End Latency: {total_time:.4f} seconds")
logging.info(f"Throughput: {throughput:.4f} tokens/s")
logging.info(f"Memory Usage: {memory_allocated:.2f} %")
logging.info(f"Power Consumption: {power_usage if power_usage is not None else 'N/A'} W")

# Print generated responses for the user
# print(response_batch)
